<?php
namespace Pinpoint\DeliveryDate\Controller\Adminhtml\Disallowed;

use Magento\Backend\App\Action;
use Pinpoint\DeliveryDate\Model\Page;
use Magento\Framework\App\Request\DataPersistorInterface;
use Magento\Framework\Exception\LocalizedException;

/**
 * Class Save
 * @package Pinpoint\DeliveryDate\Controller\Adminhtml\Disallowed
 */
class Save extends \Magento\Backend\App\Action
{
    /**
     * Authorization level of a basic admin session
     *
     * @see _isAllowed()
     */
    const ADMIN_RESOURCE = 'Pinpoint_DeliveryDate::disallowedDate';

    /**
     * @var DataPersistorInterface
     */
    protected $dataPersistor;

    /**
     * @var \Pinpoint\DeliveryDate\Model\DisallowedDate
     */
    protected $disallowedDate;

    /**
     * @param Action\Context $context
     * @param DataPersistorInterface $dataPersistor
     */
    public function __construct(
        Action\Context $context,
        DataPersistorInterface $dataPersistor,
        \Pinpoint\DeliveryDate\Model\DisallowedDate $disallowedDate
    ) {
        $this->dataPersistor = $dataPersistor;
        $this->disallowedDate = $disallowedDate;
        parent::__construct($context);
    }

    /**
     * Save action
     *
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     * @return \Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        $data = $this->getRequest()->getPostValue();
        /** @var \Magento\Backend\Model\View\Result\Redirect $resultRedirect */
        $resultRedirect = $this->resultRedirectFactory->create();
        if ($data) {
            if (empty($data['id'])) {
                $data['id'] = null;
            }

            /** @var \Pinpoint\DeliveryDate\Model\DisallowedDate $model */
            $model = $this->disallowedDate;

            $id = $this->getRequest()->getParam('id');
            if ($id) {
                $model->load($id);
            }

            $model->setData($data);

            try {
                $model->save();
                $this->messageManager->addSuccess(__('Date has been saved.'));
                $this->dataPersistor->clear('pinpoint_deliverydate_disalloweddate');
                if ($this->getRequest()->getParam('back')) {
                    return $resultRedirect->setPath('*/*/edit', ['id' => $model->getId(), '_current' => true]);
                }
                return $resultRedirect->setPath('*/*/list');
            } catch (LocalizedException $e) {
                $this->messageManager->addError($e->getMessage());
            } catch (\Exception $e) {
                $this->messageManager->addException($e, __('Something went wrong while saving the date.'));
            }

            $this->dataPersistor->set('pinpoint_deliverydate_disalloweddate', $data);
            return $resultRedirect->setPath('*/*/edit', ['id' => $this->getRequest()->getParam('id')]);
        }
        return $resultRedirect->setPath('*/*/list');
    }    
}
