<?php
namespace Pinpoint\DeliveryDate\Model;
/**
 * Class EstimatedDate
 * @package Pinpoint\DeliveryDate\Model
 */
class EstimatedDate extends \Magento\Framework\Model\AbstractModel implements \Pinpoint\DeliveryDate\Api\Data\EstimatedDateInterface, \Magento\Framework\DataObject\IdentityInterface
{
    /**
     * @var string
     */
    const CACHE_TAG = 'pinpoint_deliverydate_estimateddate';

    /**
     * @var string
     */
    const DATE_FORMAT_CONFIG_PATH = 'pinpoint_deliverydate/deliverydate/date_format';

    /**
     * @var DisallowedDate
     */
    protected $disallowedDate;

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * EstimatedDate constructor.
     * @param \Magento\Framework\Model\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param DisallowedDate $disallowedDate
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Magento\Framework\Model\ResourceModel\AbstractResource|null $resource
     * @param \Magento\Framework\Data\Collection\AbstractDb|null $resourceCollection
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        DisallowedDate $disallowedDate,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = []
    ) {
        $this->disallowedDate = $disallowedDate;
        $this->scopeConfig = $scopeConfig;
        parent::__construct($context, $registry, $resource, $resourceCollection, $data);
    }

    protected function _construct()
    {
        $this->_init('Pinpoint\DeliveryDate\Model\ResourceModel\EstimatedDate');
    }

    /**
     * @return array
     */
    public function getIdentities()
    {
        return [self::CACHE_TAG . '_' . $this->getId()];
    }

    /**
     * @param $shippingCode
     * @param int $store
     * @return false|string
     */
    public function getDateByShippingCode($shippingCode, $store = 0)
    {
        if ($shippingCode == "") {
            return 'N/A';
        }

        $estimation = $this->getCollection()
            ->addFieldToFilter('shipping_method_code', ['like' => '%'.$shippingCode.'%'])
            ->addFieldToFilter('store_id', $store)
            ->getFirstItem()
        ;

        $disallowedDates = $this->disallowedDate->getCollection()->addFieldToSelect('disallowed_date')->getItems();

        if ($estimation) {
            $dates = [];

            foreach ($disallowedDates as $date) {
                $dates[] = $date['disallowed_date'];
            }

            $excludingWeekdays = explode(',', $estimation->getData('excluding_weekdays'));

            if (is_int(array_search("0", $excludingWeekdays))) {
                $excludingWeekdays[array_search("0", $excludingWeekdays)] = 7;
            }

            return $this->getDeliveryDate(
                (int) $estimation->getData('estimated_delivery_days'),
                null,
                $dates,
                $excludingWeekdays
            );
        }

        return 'N/A';
    }

    /**
     * @param $num_business_days
     * @param null $today_ymd
     * @param array $holiday_dates_ymd
     * @param array $excludingDays
     * @return false|string
     */
    protected function getDeliveryDate($num_business_days, $today_ymd = null, $holiday_dates_ymd = [], $excludingDays = []) {
        $business_day_count = 0;
        $current_timestamp = empty($today_ymd) ? time() : strtotime($today_ymd);

        while ($business_day_count < $num_business_days) {
            $next1WD = strtotime('+1 day', $current_timestamp);
            $next1WDDate = date('Y-m-d', $next1WD);
            $dayNumber = date('N', $next1WD);
            if (!in_array($next1WDDate, $holiday_dates_ymd) && !in_array($dayNumber, $excludingDays)) {
                $business_day_count++;
            }
            $current_timestamp = $next1WD;
        }

        return date(
            $this->scopeConfig->getValue(self::DATE_FORMAT_CONFIG_PATH, \Magento\Store\Model\ScopeInterface::SCOPE_STORE),
            $current_timestamp
        );
    }
}